"""
blue_chip.tasks.linting
~~~~~~~~~~~~~~~~~~~~~~~
"""
import pathlib
from shutil import get_terminal_size

from invoke import task

from blue_chip import config, constants

__all__ = ["cfg_lint", "lint"]


STAR_SEP = "*" * get_terminal_size().columns


@task
def cfg_lint(ctx, force=False):  # pylint:disable=unused-argument
    """Configure prospector profiles."""
    if constants.BC_LINTRC_PATH.exists() and not force:
        return
    lintrc_path = pathlib.Path(constants.BC_LINTRC_PATH)
    lintrc_path.mkdir(exist_ok=True)
    for profile_name, profile_content in config.data.LINT_DATA.items():
        profile_path = lintrc_path / profile_name
        print(f"  Initializing {profile_name} ...")
        with open(profile_path, mode="w", encoding="utf-8") as f_out:
            f_out.write(profile_content)
    print(
        "Lint configuration profiles created at:\n",
        f"\t{constants.BC_LINTRC_PATH}\n{STAR_SEP}",
    )


@task(pre=[cfg_lint])
def lint(ctx, targets="."):
    """Run static analysis on python source code."""
    args = [
        "prospector",
        "--profile-path",
        str(constants.BC_LINTRC_PATH),
        "--profile",
        "bc_default",
    ]
    ctx.run(f"{' '.join(args)} {targets}")



"""
blue_chip.config.data.py
~~~~~~~~~~~~~~~~~~~~~~~~
"""
from blue_chip.constants import LINE_LENGTH

LINT_DATA = {
    "bc_root.yaml": f"""pylint:
  disable:
    # covered by pep8
    - line-too-long
    # black conflict
    - bad-continuation

pep8:
  options:
    max-line-length: {LINE_LENGTH}

pep257:
  disable:
    - D400
    # First line rules conflict with sphinx requirements.
    - D415
    # blank line required between summary line and description
    - D205
    # Multi-line docstring summary should start at the first line
    - D212
""",
    "bc_audit.yaml": """strictness: veryhigh
test-warnings: false
doc-warnings: true

inherits:
  - bc_root.yaml

pylint:
  options:
    # max-args default = 5
    max-args: 4
    # max-locals default = 15
    max-locals: 12
    # max-branches default = 15
    max-branches: 12

mccabe:
  options:
    max-complexity: 7

pyroma:
  run: True
  disable:
    - PYR18
""",
    "bc_default.yaml": """strictness: veryhigh
test-warnings: false
doc-warnings: true

inherits:
  - bc_audit.yaml

pylint:
  options:
    # max-args default = 5
    max-args: 6
    # max-locals default = 15
    max-locals: 15
    # max-branches default = 15
    max-branches: 15
    good-names: E,i,j,d,k,v,_
  disable:
    - locally-disabled
    - fixme

pyflakes:
  disable:
    # covered by pylint
    - F841

mccabe:
  options:
    max-complexity: 10

pep257:
  run: True
""",
    "bc_tests.yaml": """inherits:
  - bc_default.yaml

strictness: high
test-warnings: true
doc-warnings: false

pyroma:
  run: False

pep257:
  run: False
""",
}



"""
blue_chip.tasks.formatting.py
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
from typing import List, Union

from invoke import task

from blue_chip import constants

__all__ = ["sort", "fmt", "fmt_only"]


@task(
    help={
        "line-length": "How many characters per line to allow."
        f" [default: { constants.LINE_LENGTH}]",
        "targets": "Paths/directories to format. [default: . ]",
    },
)
def sort(ctx, line_length=constants.LINE_LENGTH, targets=".", check=False):
    """Sort module imports."""
    print("sorting imports ...")
    args = [
        "isort",
        "--profile",
        "black",
        "-l",
        str(line_length),
        targets,
    ]
    if check:
        args.append("--check-only")
    ctx.run(" ".join(args))


def _fmt_cmd(line_length: int, targets: Union[str, List[str]], check=False) -> str:
    args = ["black", "--line-length", str(line_length)]
    if check:
        args.append("--check")
    if isinstance(targets, (list, tuple, set)):
        args.extend(targets)
    else:
        args.append(targets)
    return " ".join(args)


@task(
    pre=[sort],
    help={
        "line-length": "How many characters per line to allow."
        f" [default: {constants.LINE_LENGTH}]",
        "targets": "Paths/directories to format. [default: . ]",
    },
)
def fmt(ctx, line_length=constants.LINE_LENGTH, targets=".", check=False):
    """Format python source code & sort imports."""
    print("formatting ...")
    ctx.run(_fmt_cmd(line_length, targets, check))


@task(
    help={
        "line-length": "How many characters per line to allow."
        f" [default: {constants.LINE_LENGTH}]",
        "targets": "Paths/directories to format. [default: . ]",
    },
)
def fmt_only(ctx, line_length=constants.LINE_LENGTH, targets="."):
    """Format python source code."""
    ctx.run(_fmt_cmd(line_length, targets))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['blue_chip', 'blue_chip.config', 'blue_chip.tasks']

package_data = \
{'': ['*']}

install_requires = \
['black>=22.3.0,<23.0.0',
 'invoke',
 'isort>=5.10.1,<6.0.0',
 'prospector[with_pyroma]>=1.2,<2.0']

entry_points = \
{'console_scripts': ['bc = blue_chip.__main__:program.run',
                     'bcp = blue_chip.__main__:program.run']}

setup_kwargs = {
    'name': 'blue-chip',
    'version': '0.0.9',
    'description': 'One click Python code quality package',
    'long_description': "[![PyPI version](https://badge.fury.io/py/blue-chip.svg)](https://badge.fury.io/py/blue-chip)\n[![Build Status](https://travis-ci.com/Kilo59/blue-chip.svg?branch=master)](https://travis-ci.com/Kilo59/blue-chip)\n[![Coverage Status](https://coveralls.io/repos/github/Kilo59/blue-chip/badge.svg?branch=master)](https://coveralls.io/github/Kilo59/blue-chip?branch=master)\n[![Language grade: Python](https://img.shields.io/lgtm/grade/python/g/Kilo59/blue-chip.svg?logo=lgtm&logoWidth=18)](https://lgtm.com/projects/g/Kilo59/blue-chip/context:python)\n[![Total alerts](https://img.shields.io/lgtm/alerts/g/Kilo59/blue-chip.svg?logo=lgtm&logoWidth=18)](https://lgtm.com/projects/g/Kilo59/blue-chip/alerts/)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n\n# blue-chip\nOne click Python code quality package\n\n`blue-chip` uses [`black`](https://github.com/ambv/black), [`isort`](https://github.com/timothycrosley/isort), [`invoke`](http://www.pyinvoke.org/) and [`prospector`](https://prospector.readthedocs.io/en/master/) configured to work well together out of the box.\n\n-----------------------\n## Install\n\nThe recommended installation method when installing globally is to use [`pipx`](https://github.com/pipxproject/pipx)\n\n```\npipx install blue-chip\n```\nOtherwise (for best results) use `pip` to install directly into your project's virtual environment.\n```\npip install blue-chip\n```\nUsing [`poetry`](https://python-poetry.org/).\n```\npoetry add --dev blue-chip\n```\nUsing [`pipenv`](https://pipenv.pypa.io/en/latest/) you need to pin the version.\n```\npipenv install --dev blue-chip==0.0.8\n```\n\n## Command-line use\n\nList the possible `blue-chip` commands with `bc --list`\n```\n$bc --list\n\nSubcommands:\n\n  fmt    Format python source code.\n  lint   Run static analysis on python source code.\n  sort   Sort module imports.\n```\n\n----------------------\n\n## Enterprise use\nEnterprise teams may find it useful to fork and redistribute this package with their own custom defined standards.\n\n\nIn many cases this is a simple as changing the package name and values in [`blue_chip/constants.py`](https://github.com/Kilo59/blue-chip/blob/master/blue_chip/constants.py).\n\nFor fine grain control of the static analysis tools, edit the `prospector` profiles in [`blue_chip/config`](https://github.com/Kilo59/blue-chip/blob/master/blue_chip/config/data.py).\n",
    'author': 'Gabriel Gore',
    'author_email': 'gabriel59kg@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/blue-chip/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



"""
blue_chip.__main__.py
~~~~~~~~~~~~~~~~~~~~~
"""
import pkg_resources
from invoke import Collection, Program

import blue_chip.tasks

PACKAGE_NAME = "blue-chip"


def get_version(pkg_name=PACKAGE_NAME):
    """
    Get the package version.

    https://github.com/python-poetry/poetry/issues/144#issuecomment-559793020
    """
    try:
        distribution = pkg_resources.get_distribution(pkg_name)
    except pkg_resources.DistributionNotFound:
        return "dev"
    else:
        return distribution.version


PKG_VERSION = get_version()

program = Program(
    namespace=Collection.from_module(blue_chip.tasks),
    name=PACKAGE_NAME,
    version=PKG_VERSION,
)
